import boto3
import os
from pathlib import Path
import re


class mint_s3(object):
    def __init__(self, access_key, secret_key, bucket_name, instance_id=1):
        # Versioning
        self.__classname__ = "MintS3"
        self.__author__ = "n.phantawee@gmail.com"
        self.__version__ = 1.00
        self.__changelog__ = {"1.00": "Create the wraper class"}
        self.access_key = access_key
        self.secret_key = secret_key
        self.bucket_name = bucket_name
        self.client = boto3.client(
            "s3",
            aws_access_key_id=self.access_key,
            aws_secret_access_key=self.secret_key,
        )

    def list_file(self, prefix):
        resp = self.client.list_objects_v2(
            Bucket=self.bucket_name, Prefix=prefix, Delimiter="/"
        )
        contents = resp.get("Contents")
        if contents is None:
            files = []
            print(f"There is no file containing [{prefix}]")
        else:
            files = [f.get("Key") for f in contents]
            files = [f for f in files if not f.endswith("/")]
        return files

    def download_file(self, blob_name, local_file_path=None):
        """
        download blob to local path
        """
        if local_file_path is None:
            local_file_path = blob_name
            local_dir, local_file = os.path.split(local_file_path)
            if len(local_dir) > 0:
                if local_dir[0] == "/":
                    local_dir = local_dir[1:]
            _bucket_name = self.remove_special_char(self.bucket_name)
            _bucket_name = f"./{_bucket_name}"
            local_dir = os.path.join(_bucket_name, local_dir)
            local_file_path = os.path.join(local_dir, local_file)
            result = self.create_folder(local_dir)
        try:
            self.client.download_file(
                self.bucket_name, blob_name, local_file_path
            )
        except Exception as e:
            print(f"Failed to download {blob_name}: {e}")
            return False
        else:
            print(f"The file {blob_name} has been downloaded")
            return True

    def create_folder(self, path):
        try:
            p = Path(path)
            p.mkdir(parents=True, exist_ok=True)
        except Exception as e:
            print(f"Failed to cteate folder {path}")
            return False
        else:
            return True

    def remove_special_char(self, text):
        """
        replace special character with _
        """
        x = re.sub("[^A-Za-zก-๐ ]+", "_", text)
        return x
